# Copyright (c) 2024 SUSE LLC, All rights reserved.
#
# This file is part of obs-img-utils. obs-img-utils provides
# an api and command line utilities for images in OBS.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pytest

from obs_img_utils.rpm import compare_rpm_labels


@pytest.mark.parametrize(
    "a_version_tag, b_version_tag, expected_output",
    [
        ('1.0', '1.0', 0),
        ('1.0', '2.0', -1),
        ('2.0', '1.0', 1),
        ('2.0.1', '2.0.1', 0),
        ('2.0', '2.0.1', -1),
        ('2.0.1', '2.0', 1),
        ('2.0.1a', '2.0.1a', 0),
        ('2.0.1a', '2.0.1', 1),
        ('2.0.1', '2.0.1a', -1),
        ('5.5p1', '5.5p1', 0),
        ('5.5p1', '5.5p2', -1),
        ('5.5p2', '5.5p1', 1),
        ('5.5p10', '5.5p10', 0),
        ('5.5p1', '5.5p10', -1),
        ('5.5p10', '5.5p1', 1),
        ('10xyz', '10.1xyz', -1),
        ('10.1xyz', '10xyz', 1),
        ('xyz10', 'xyz10', 0),
        ('xyz10', 'xyz10.1', -1),
        ('xyz10.1', 'xyz10', 1),
        ('xyz.4', 'xyz.4', 0),
        ('xyz.4', '8', -1),
        ('8', 'xyz.4', 1),
        ('xyz.4', '2', -1),
        ('2', 'xyz.4', 1),
        ('5.5p2', '5.6p1', -1),
        ('5.6p1', '5.5p2', 1),
        ('5.6p1', '6.5p1', -1),
        ('6.5p1', '5.6p1', 1),
        ('6.0.rc1', '6.0', 1),
        ('6.0', '6.0.rc1', -1),
        ('1.0', '2.0', -1),
        ('2.0', '1.0', 1),
        ('2.0.1', '2.0.1', 0),
        ('2.0', '2.0.1', -1),
        ('2.0.1', '2.0', 1),
        ('2.0.1a', '2.0.1a', 0),
        ('2.0.1a', '2.0.1', 1),
        ('2.0.1', '2.0.1a', -1),
        ('5.5p1', '5.5p1', 0),
        ('5.5p1', '5.5p2', -1),
        ('5.5p2', '5.5p1', 1),
        ('5.5p10', '5.5p10', 0),
        ('5.5p1', '5.5p10', -1),
        ('5.5p10', '5.5p1', 1),
        ('10xyz', '10.1xyz', -1),
        ('10.1xyz', '10xyz', 1),
        ('xyz10', 'xyz10', 0),
        ('xyz10', 'xyz10.1', -1),
        ('xyz10.1', 'xyz10', 1),
        ('xyz.4', 'xyz.4', 0),
        ('xyz.4', '8', -1),
        ('8', 'xyz.4', 1),
        ('xyz.4', '2', -1),
        ('2', 'xyz.4', 1),
        ('5.5p2', '5.6p1', -1),
        ('5.6p1', '5.5p2', 1),
        ('5.6p1', '6.5p1', -1),
        ('6.5p1', '5.6p1', 1),
        ('6.0.rc1', '6.0', 1),
        ('6.0', '6.0.rc1', -1),
        ('10b2', '10a1', 1),
        ('10a2', '10b2', -1),
        ('1.0aa', '1.0aa', 0),
        ('1.0a', '1.0aa', -1),
        ('1.0aa', '1.0a', 1),
        ('10.0001', '10.0001', 0),
        ('10.0001', '10.1', 0),
        ('10.1', '10.0001', 0),
        ('10.0001', '10.0039', -1),
        ('10.0039', '10.0001', 1),
        ('4.999.9', '5.0', -1),
        ('5.0', '4.999.9', 1),
        ('20101121', '20101121', 0),
        ('20101121', '20101122', -1),
        ('20101122', '20101121', 1),
        ('2_0', '2_0', 0),
        ('2.0', '2_0', 0),
        ('2_0', '2.0', 0),
        ('a', 'a', 0),
        ('a+', 'a+', 0),
        ('a+', 'a_', 0),
        ('a_', 'a+', 0),
        ('+a', '+a', 0),
        ('+a', '_a', 0),
        ('_a', '+a', 0),
        ('+_', '+_', 0),
        ('_+', '+_', 0),
        ('_+', '_+', 0),
        ('+', '_', 0),
        ('_', '+', 0),
        ('1.0~rc1', '1.0~rc1', 0),
        ('1.0~rc1', '1.0', -1),
        ('1.0', '1.0~rc1', 1),
        ('1.0~rc1', '1.0~rc2', -1),
        ('1.0~rc2', '1.0~rc1', 1),
        ('1.0~rc1~git123', '1.0~rc1~git123', 0),
        ('1.0~rc1~git123', '1.0~rc1', -1),
        ('1.0~rc1', '1.0~rc1~git123', 1),
        ('1.0^', '1.0^', 0),
        ('1.0^', '1.0', 1),
        ('1.0', '1.0^', -1),
        ('1.0^git1', '1.0^git1', 0),
        ('1.0^git1', '1.0', 1),
        ('1.0', '1.0^git1', -1),
        ('1.0^git1', '1.0^git2', -1),
        ('1.0^git2', '1.0^git1', 1),
        ('1.0^git1', '1.01', -1),
        ('1.01', '1.0^git1', 1),
        ('1.0^20160101', '1.0^20160101', 0),
        ('1.0^20160101', '1.0.1', -1),
        ('1.0.1', '1.0^20160101', 1),
        ('1.0^20160101^git1', '1.0^20160101^git1', 0),
        ('1.0^20160102', '1.0^20160101^git1', 1),
        ('1.0^20160101^git1', '1.0^20160102', -1),
        ('1.0~rc1^git1', '1.0~rc1^git1', 0),
        ('1.0~rc1^git1', '1.0~rc1', 1),
        ('1.0~rc1', '1.0~rc1^git1', -1),
        ('1.0^git1~pre', '1.0^git1~pre', 0),
        ('1.0^git1', '1.0^git1~pre', 1),
        ('1.0^git1~pre', '1.0^git1', -1),
        ('1b.fc17', '1b.fc17', 0),
        ('1b.fc17', '1.fc17', -1),
        ('1.fc17', '1b.fc17', 1),
        ('1g.fc17', '1g.fc17', 0),
        ('1g.fc17', '1.fc17', 1),
        ('1.fc17', '1g.fc17', -1),
        ('1.1.α', '1.1.α', 0),
        ('1.1.α', '1.1.β', 0),
        ('1.1.β', '1.1.α', 0),
        ('1.1.αα', '1.1.α', 0),
        ('1.1.α', '1.1.ββ', 0),
        ('1.1.ββ', '1.1.αα', 0),
        ('2~almost', '2.0.1', -1),
    ]
)
def test_compare_rpm_labels(a_version_tag, b_version_tag, expected_output):
    assert compare_rpm_labels(
        (0, a_version_tag, '0'),
        (0, b_version_tag, '0'),
    ) == expected_output
